# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tensorflow/contrib/tensorboard/graph_explorer/proto/graph_explorer.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='tensorflow/contrib/tensorboard/graph_explorer/proto/graph_explorer.proto',
  package='graph_explorer',
  syntax='proto3',
  serialized_pb=_b('\nHtensorflow/contrib/tensorboard/graph_explorer/proto/graph_explorer.proto\x12\x0egraph_explorer\"\xf0\x01\n\x04Node\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x35\n\tnode_attr\x18\x02 \x03(\x0b\x32\".graph_explorer.Node.NodeAttrEntry\x12=\n\rmetanode_attr\x18\x03 \x03(\x0b\x32&.graph_explorer.Node.MetanodeAttrEntry\x1a/\n\rNodeAttrEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a\x33\n\x11MetanodeAttrEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\x8e\x01\n\x04\x45\x64ge\x12\x0e\n\x06source\x18\x01 \x01(\t\x12\x0e\n\x06target\x18\x02 \x01(\t\x12\x35\n\tedge_attr\x18\x03 \x03(\x0b\x32\".graph_explorer.Edge.EdgeAttrEntry\x1a/\n\rEdgeAttrEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\x82\x04\n\x05Graph\x12\"\n\x04node\x18\x01 \x03(\x0b\x32\x14.graph_explorer.Node\x12\"\n\x04\x65\x64ge\x18\x02 \x03(\x0b\x32\x14.graph_explorer.Edge\x12\x36\n\tnode_attr\x18\x03 \x03(\x0b\x32#.graph_explorer.Graph.NodeAttrEntry\x12>\n\rmetanode_attr\x18\x04 \x03(\x0b\x32\'.graph_explorer.Graph.MetanodeAttrEntry\x12\x36\n\tedge_attr\x18\x05 \x03(\x0b\x32#.graph_explorer.Graph.EdgeAttrEntry\x12\x38\n\ngraph_attr\x18\x06 \x03(\x0b\x32$.graph_explorer.Graph.GraphAttrEntry\x1a/\n\rNodeAttrEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a\x33\n\x11MetanodeAttrEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a/\n\rEdgeAttrEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a\x30\n\x0eGraphAttrEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x62\x06proto3')
)




_NODE_NODEATTRENTRY = _descriptor.Descriptor(
  name='NodeAttrEntry',
  full_name='graph_explorer.Node.NodeAttrEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='graph_explorer.Node.NodeAttrEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='value', full_name='graph_explorer.Node.NodeAttrEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=_descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('8\001')),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=233,
  serialized_end=280,
)

_NODE_METANODEATTRENTRY = _descriptor.Descriptor(
  name='MetanodeAttrEntry',
  full_name='graph_explorer.Node.MetanodeAttrEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='graph_explorer.Node.MetanodeAttrEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='value', full_name='graph_explorer.Node.MetanodeAttrEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=_descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('8\001')),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=282,
  serialized_end=333,
)

_NODE = _descriptor.Descriptor(
  name='Node',
  full_name='graph_explorer.Node',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='graph_explorer.Node.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='node_attr', full_name='graph_explorer.Node.node_attr', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='metanode_attr', full_name='graph_explorer.Node.metanode_attr', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[_NODE_NODEATTRENTRY, _NODE_METANODEATTRENTRY, ],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=93,
  serialized_end=333,
)


_EDGE_EDGEATTRENTRY = _descriptor.Descriptor(
  name='EdgeAttrEntry',
  full_name='graph_explorer.Edge.EdgeAttrEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='graph_explorer.Edge.EdgeAttrEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='value', full_name='graph_explorer.Edge.EdgeAttrEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=_descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('8\001')),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=431,
  serialized_end=478,
)

_EDGE = _descriptor.Descriptor(
  name='Edge',
  full_name='graph_explorer.Edge',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='source', full_name='graph_explorer.Edge.source', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='target', full_name='graph_explorer.Edge.target', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='edge_attr', full_name='graph_explorer.Edge.edge_attr', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[_EDGE_EDGEATTRENTRY, ],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=336,
  serialized_end=478,
)


_GRAPH_NODEATTRENTRY = _descriptor.Descriptor(
  name='NodeAttrEntry',
  full_name='graph_explorer.Graph.NodeAttrEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='graph_explorer.Graph.NodeAttrEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='value', full_name='graph_explorer.Graph.NodeAttrEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=_descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('8\001')),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=233,
  serialized_end=280,
)

_GRAPH_METANODEATTRENTRY = _descriptor.Descriptor(
  name='MetanodeAttrEntry',
  full_name='graph_explorer.Graph.MetanodeAttrEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='graph_explorer.Graph.MetanodeAttrEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='value', full_name='graph_explorer.Graph.MetanodeAttrEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=_descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('8\001')),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=282,
  serialized_end=333,
)

_GRAPH_EDGEATTRENTRY = _descriptor.Descriptor(
  name='EdgeAttrEntry',
  full_name='graph_explorer.Graph.EdgeAttrEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='graph_explorer.Graph.EdgeAttrEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='value', full_name='graph_explorer.Graph.EdgeAttrEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=_descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('8\001')),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=431,
  serialized_end=478,
)

_GRAPH_GRAPHATTRENTRY = _descriptor.Descriptor(
  name='GraphAttrEntry',
  full_name='graph_explorer.Graph.GraphAttrEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='graph_explorer.Graph.GraphAttrEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='value', full_name='graph_explorer.Graph.GraphAttrEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=_descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('8\001')),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=947,
  serialized_end=995,
)

_GRAPH = _descriptor.Descriptor(
  name='Graph',
  full_name='graph_explorer.Graph',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='node', full_name='graph_explorer.Graph.node', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='edge', full_name='graph_explorer.Graph.edge', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='node_attr', full_name='graph_explorer.Graph.node_attr', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='metanode_attr', full_name='graph_explorer.Graph.metanode_attr', index=3,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='edge_attr', full_name='graph_explorer.Graph.edge_attr', index=4,
      number=5, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='graph_attr', full_name='graph_explorer.Graph.graph_attr', index=5,
      number=6, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[_GRAPH_NODEATTRENTRY, _GRAPH_METANODEATTRENTRY, _GRAPH_EDGEATTRENTRY, _GRAPH_GRAPHATTRENTRY, ],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=481,
  serialized_end=995,
)

_NODE_NODEATTRENTRY.containing_type = _NODE
_NODE_METANODEATTRENTRY.containing_type = _NODE
_NODE.fields_by_name['node_attr'].message_type = _NODE_NODEATTRENTRY
_NODE.fields_by_name['metanode_attr'].message_type = _NODE_METANODEATTRENTRY
_EDGE_EDGEATTRENTRY.containing_type = _EDGE
_EDGE.fields_by_name['edge_attr'].message_type = _EDGE_EDGEATTRENTRY
_GRAPH_NODEATTRENTRY.containing_type = _GRAPH
_GRAPH_METANODEATTRENTRY.containing_type = _GRAPH
_GRAPH_EDGEATTRENTRY.containing_type = _GRAPH
_GRAPH_GRAPHATTRENTRY.containing_type = _GRAPH
_GRAPH.fields_by_name['node'].message_type = _NODE
_GRAPH.fields_by_name['edge'].message_type = _EDGE
_GRAPH.fields_by_name['node_attr'].message_type = _GRAPH_NODEATTRENTRY
_GRAPH.fields_by_name['metanode_attr'].message_type = _GRAPH_METANODEATTRENTRY
_GRAPH.fields_by_name['edge_attr'].message_type = _GRAPH_EDGEATTRENTRY
_GRAPH.fields_by_name['graph_attr'].message_type = _GRAPH_GRAPHATTRENTRY
DESCRIPTOR.message_types_by_name['Node'] = _NODE
DESCRIPTOR.message_types_by_name['Edge'] = _EDGE
DESCRIPTOR.message_types_by_name['Graph'] = _GRAPH
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Node = _reflection.GeneratedProtocolMessageType('Node', (_message.Message,), dict(

  NodeAttrEntry = _reflection.GeneratedProtocolMessageType('NodeAttrEntry', (_message.Message,), dict(
    DESCRIPTOR = _NODE_NODEATTRENTRY,
    __module__ = 'tensorflow.contrib.tensorboard.graph_explorer.proto.graph_explorer_pb2'
    # @@protoc_insertion_point(class_scope:graph_explorer.Node.NodeAttrEntry)
    ))
  ,

  MetanodeAttrEntry = _reflection.GeneratedProtocolMessageType('MetanodeAttrEntry', (_message.Message,), dict(
    DESCRIPTOR = _NODE_METANODEATTRENTRY,
    __module__ = 'tensorflow.contrib.tensorboard.graph_explorer.proto.graph_explorer_pb2'
    # @@protoc_insertion_point(class_scope:graph_explorer.Node.MetanodeAttrEntry)
    ))
  ,
  DESCRIPTOR = _NODE,
  __module__ = 'tensorflow.contrib.tensorboard.graph_explorer.proto.graph_explorer_pb2'
  # @@protoc_insertion_point(class_scope:graph_explorer.Node)
  ))
_sym_db.RegisterMessage(Node)
_sym_db.RegisterMessage(Node.NodeAttrEntry)
_sym_db.RegisterMessage(Node.MetanodeAttrEntry)

Edge = _reflection.GeneratedProtocolMessageType('Edge', (_message.Message,), dict(

  EdgeAttrEntry = _reflection.GeneratedProtocolMessageType('EdgeAttrEntry', (_message.Message,), dict(
    DESCRIPTOR = _EDGE_EDGEATTRENTRY,
    __module__ = 'tensorflow.contrib.tensorboard.graph_explorer.proto.graph_explorer_pb2'
    # @@protoc_insertion_point(class_scope:graph_explorer.Edge.EdgeAttrEntry)
    ))
  ,
  DESCRIPTOR = _EDGE,
  __module__ = 'tensorflow.contrib.tensorboard.graph_explorer.proto.graph_explorer_pb2'
  # @@protoc_insertion_point(class_scope:graph_explorer.Edge)
  ))
_sym_db.RegisterMessage(Edge)
_sym_db.RegisterMessage(Edge.EdgeAttrEntry)

Graph = _reflection.GeneratedProtocolMessageType('Graph', (_message.Message,), dict(

  NodeAttrEntry = _reflection.GeneratedProtocolMessageType('NodeAttrEntry', (_message.Message,), dict(
    DESCRIPTOR = _GRAPH_NODEATTRENTRY,
    __module__ = 'tensorflow.contrib.tensorboard.graph_explorer.proto.graph_explorer_pb2'
    # @@protoc_insertion_point(class_scope:graph_explorer.Graph.NodeAttrEntry)
    ))
  ,

  MetanodeAttrEntry = _reflection.GeneratedProtocolMessageType('MetanodeAttrEntry', (_message.Message,), dict(
    DESCRIPTOR = _GRAPH_METANODEATTRENTRY,
    __module__ = 'tensorflow.contrib.tensorboard.graph_explorer.proto.graph_explorer_pb2'
    # @@protoc_insertion_point(class_scope:graph_explorer.Graph.MetanodeAttrEntry)
    ))
  ,

  EdgeAttrEntry = _reflection.GeneratedProtocolMessageType('EdgeAttrEntry', (_message.Message,), dict(
    DESCRIPTOR = _GRAPH_EDGEATTRENTRY,
    __module__ = 'tensorflow.contrib.tensorboard.graph_explorer.proto.graph_explorer_pb2'
    # @@protoc_insertion_point(class_scope:graph_explorer.Graph.EdgeAttrEntry)
    ))
  ,

  GraphAttrEntry = _reflection.GeneratedProtocolMessageType('GraphAttrEntry', (_message.Message,), dict(
    DESCRIPTOR = _GRAPH_GRAPHATTRENTRY,
    __module__ = 'tensorflow.contrib.tensorboard.graph_explorer.proto.graph_explorer_pb2'
    # @@protoc_insertion_point(class_scope:graph_explorer.Graph.GraphAttrEntry)
    ))
  ,
  DESCRIPTOR = _GRAPH,
  __module__ = 'tensorflow.contrib.tensorboard.graph_explorer.proto.graph_explorer_pb2'
  # @@protoc_insertion_point(class_scope:graph_explorer.Graph)
  ))
_sym_db.RegisterMessage(Graph)
_sym_db.RegisterMessage(Graph.NodeAttrEntry)
_sym_db.RegisterMessage(Graph.MetanodeAttrEntry)
_sym_db.RegisterMessage(Graph.EdgeAttrEntry)
_sym_db.RegisterMessage(Graph.GraphAttrEntry)


_NODE_NODEATTRENTRY.has_options = True
_NODE_NODEATTRENTRY._options = _descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('8\001'))
_NODE_METANODEATTRENTRY.has_options = True
_NODE_METANODEATTRENTRY._options = _descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('8\001'))
_EDGE_EDGEATTRENTRY.has_options = True
_EDGE_EDGEATTRENTRY._options = _descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('8\001'))
_GRAPH_NODEATTRENTRY.has_options = True
_GRAPH_NODEATTRENTRY._options = _descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('8\001'))
_GRAPH_METANODEATTRENTRY.has_options = True
_GRAPH_METANODEATTRENTRY._options = _descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('8\001'))
_GRAPH_EDGEATTRENTRY.has_options = True
_GRAPH_EDGEATTRENTRY._options = _descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('8\001'))
_GRAPH_GRAPHATTRENTRY.has_options = True
_GRAPH_GRAPHATTRENTRY._options = _descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('8\001'))
# @@protoc_insertion_point(module_scope)
